
function [bic,khat,bmat,bmat1]=brkEst(y,x,nbrk,est_brk,dmin);
%  estimates the break by picking the one with the lowest SSR from all possible
%  breaks
% AFTER STOCK AND WATSON'S GAUSS PROCEDURE  2/13/03
% @---------------------------------------------------------------------------@
% @            BRKEST.PRC     JHS 11/29/95                                    @
% @---------------------------------------------------------------------------@
% /*   estimation of multiple break points:
% Break in CONSTANT! term in regression function ONLY
% x = matrix of regressors, EXCLUDING constant
% y = vector of rhv
% nbrk = number of breaks to estimate
% dmin = minimal fraction of data set separating estimated breaks
%
% Output:     bic     value of BIC at each break
%             khat    vector of break dates
%             bmat    length(y) x nbr+1 matrix of  break regressors (dummies) incl constant
% */
if min(var(x))==0; error('Do not include a constant in the design matrix x'); end;
enY=length(y);
if est_brk==1;  %@ estimate break @
	tt=(1:enY)';
	tmin=ceil(dmin*enY);
	tt(1:tmin)=zeros(tmin,1); tt((end-tmin+1):end)=zeros(tmin,1);
	syy=y'*y; bic=zeros(nbrk,1); khat=bic;
	xi=[ones(enY,1) x];
	for i=1:nbrk;
		sxxi=xi'*xi; syxi=xi'*y;
		ttsel=selif(tt,(tt>0)); lssr=zeros(size(ttsel));
		bik=ones(enY,1);
		sxxik=zeros(size(sxxi,1)+1);
		sxxik(2:end,2:end)=sxxi;
		syxik=[0;syxi];
		for k=1:length(ttsel); 
            tk=ttsel(k);
			bik(1:(tk-1))=zeros(tk-1,1);
			sxxik(1,:)=bik'*[bik xi];
			sxxik(:,1)=sxxik(1,:)';
			syxik(1)=bik'*y;
			lssr(k)=log( (syy-syxik'*invpd(sxxik)*syxik)/rows(y) );
        end;
        [mlssr,ilssr]= min(lssr); 
        %figure; plot(exp(lssr))
		bic(i)=mlssr+size(sxxik,2)*log(enY)/enY; 
		khat(i)=ttsel(ilssr);
		%@ prepare for next iteration @
        bkhat=zeros(enY,1);
		bkhat(khat(i):end)=ones(length(bkhat)-khat(i)+1,1);
		xi=[bkhat xi];
		tt((khat(i)-tmin+1):(khat(i)+tmin))=zeros(2*tmin,1);
    end;
else;  %@ don't estimate break, only create final equispaced step functions @
    khat=floor(cumsum(ones(nbrk,1)*enY/(nbrk+1))); bic=0;
end;
%@ create matrix of break regressors, with first regressor a constant, using estimated break dates @
khats=sort(khat); khats=[1;khats;enY+1];
bmat=zeros(enY,length(khats)-1); 
for i=1:length(khats)-1; bmat(khats(i):(khats(i+1)-1),i)=ones(khats(i+1)-khats(i),1); end;
bmat1=bmat; bmat1(:,1)=ones(size(bmat,1),1);
