function [ci,newNRU,ARstatOut] = ciSearch5(inputCI)
%
% calculates the NAIRU CI
% inputCI:  nairuGuess    
%           nairuMin
%          nairuMax
%          nairuMesh (if meyhod = 2, should be very coarse, like 1)
%          tolerance  (when to stop efficient algorithm)
%           X   RHS variables
%           y   LHS variable
%           lags   #of lags in regression
%           arMethod  method for calculating AR stat in nairuFstat
% searchMethod 1   grid search
%            2  SSW eff method (kinda bisection-like) 
%
mmv2struct(inputCI);
splineSize=splineSize-1; % I will throw away the constant
arGrid = (nairuMin:nairuMesh:nairuMax)'; unSpread = allLags(2)-allLags(1)+1;
inputAR = mmv2struct(y,arGrid,unSpread,splineIndex,splineSize); 

chi2CritVal = chi2inv(.95,1); ci = [nan, nan]; newNRU=nan; dgrsFr = size(X,1)-(size(X,2)+1);
mMat = eye(size(X,1))- X*pinv(X'*X)*X'; % residual projection matrix
ssr0 = y'*mMat*y; X = X(:,2:end); %throw away constant

iCount = 1; ci = repmat(nan,length(timeFrame),2); %ARstat2=repmat(nan,length(timeFrame),length(arGrid));
for i = timeFrame';
	ubarNow = ubar(i); seUbarNow = seUbar(i);
   if splineIndex == 1; splinesNow = X(i,1:splineSize);
		splinesAdj = X(:,1:splineSize)-repmat(splinesNow,size(X,1),1);
      Xadj = [splinesAdj, X(:,splineSize+1:end)];
   else; Xadj = X; 
   end;
	inputAR.X=Xadj;
	switch (searchMethod);
	case 1;
		ssr1 = arSSR(inputAR);  ARstat = (ssr1-ssr0)/(ssr0/dgrsFr); ARstat2(iCount,:)=ARstat';
		ciAux = selif(arGrid,ARstat<chi2CritVal); ci(iCount,:) = [ciAux(1) ciAux(end)];
      newNRU(iCount)=selif(arGrid,ARstat==min(ARstat));
		if (ARstat(1)<chi2CritVal) | (ARstat(end)<chi2CritVal);
         warning(strcat('Grid value ',int2str(iCount), ': NAIRU range is too narrow, please decrease nairuMin and/or increase nairuMax')); 
      end;
% 		figure; h = plot(arGrid,ARstat, arGrid, repmat(chi2CritVal,length(ARstat),1),'--'); 
% 		set(h,'LineWidth',1.5); title('F statistic');%, 'Fontsize',14);
% 		xlabel('NAIRU'); ylabel('F statistic'); legend('F statistic','Crit value',4); grid on;

	case 2;
     k = 1; split = 6;   % how much divide the subintervals?
     for j = [-1,1]; 
        nairuLim = ubarNow+6*seUbarNow*[-1; 1]; 
        nairuMesh = (ubarNow-nairuLim(1))/split;
        while (nairuMesh>tolerance)|(nairuMesh>1000*seUbarNow);
           nruRange = (mySeq(ubarNow,nairuMesh,nairuLim(k)))'; inputAR.arGrid = nruRange; 
           ssr1 = arSSR(inputAR); ARstat = (ssr1-ssr0)/(ssr0/dgrsFr);
           if sum(ARstat<chi2CritVal) == size(ARstat,1); nairuMesh = nairuMesh*split;
           else; 
              ci1 = selif(nruRange,ARstat<chi2CritVal); ubarNow = sign(j)*max(j*ci1); 
              ci(iCount,k) = ubarNow; 
              nairuMesh = nairuMesh/split; 
           end; 
           nairuLim(k) = ubarNow+j*split*nairuMesh;
        end;
        k = k+1;
     end;
   end;
   if abs(ci(iCount,1))>100; ci(iCount,1)=-100; end;
   if abs(ci(iCount,2))>100; ci(iCount,2)=100; end;   
   fprintf('Run %2.0f of %2.0f    CI:  (%5.5f, %5.5f)\n',iCount,length(timeFrame),ci(iCount,1),ci(iCount,2)); iCount = iCount+1; 
end; 
if searchMethod==1 &size(timeFrame,1)>1; 
      ARstat2I = interp1(timeFrame,ARstat2,(min(timeFrame):max(timeFrame))');
      [x,y]=meshgrid(arGrid,(min(timeFrame):max(timeFrame))');
      figure; h = surf(x,y,ARstat2I); colorbar; hold on; surf(x,y,repmat(chi2CritVal,size(ARstat2I,1),size(ARstat2I,2))); hold off;
elseif searchMethod==1 &size(timeFrame,1)==1; 
	   figure; h = plot(arGrid,ARstat, arGrid, repmat(chi2CritVal,length(ARstat),1),'--'); 
		set(h,'LineWidth',1.5); title('Anderson-Rubin Statistic');%, 'Fontsize',14);
		xlabel('NAIRU (%)'); ylabel('F statistic'); legend('F statistic','Crit value',4); grid on;
end;
if nargout==3; ARstatOut=ARstat; end;

%==================================================================
function y=mySeq(lim1,limMesh,lim2);
% 
%  if lim1>lim2, reverses their order
%
if lim1<lim2; y=lim1:limMesh:lim2;
else y=lim2:limMesh:lim1; end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ssrR=arSSR(inputAR)
%
%  input:  X regresors
%          y  LHS variable
%          arGrid  sequence of H_0 nairus
%          unSpread  # of lags
%           splineIndex
%           splineSize    # splines-1  (constant's away)
%
mmv2struct(inputAR); T=length(arGrid);
if any(~sum(X~=1))==1; error('Make sure X matrix is WITHOUT constant.'); end;

if splineIndex==1; unInd=((splineSize+1):splineSize+unSpread)'; X1=X(:,1:splineSize);
else; unInd=(1:unSpread)'; X1=[]; end;
for i=1:T;
    Xr=[X1 X(:,unInd)-arGrid(i), X(:,max(unInd)+1:end)]; % restricted  matrices
    betaR=Xr\y; MmatR=eye(size(Xr,1))-Xr*pinv(Xr'*Xr)*Xr'; ssrR(i,1) = y'*MmatR*y; 
end;
