function yF=bpass(y,up,dn,K);
%  THIS IS A MODIFIED VERSION OF BAXTER AND KING'S BPF.M FILE
% bpf.m  
% Program to compute band-pass filtered series
% Inputs are
%  y:   data (rows = observations, columns=series) 
%  up:  period corresponding to highest frequency (e.g., 6)
%  dn:  period corresponding to lowest frequency (e.g., 32)
%  K:   number of terms in approximating moving average
%  [calls filtk.m (filter with symmetric weights) as subroutine]
x=[up dn]; y=makeCol(y); t=length(y);
if (up>dn); dn=x(1); up=x(2); end;
if (up<2); up=2; end;
avec=bpweight(up,dn,K); yF=repmat(nan,t,1);
for i= K+1:t-K; yF(i,1)=y(i-K:i+K)'*avec; end; 

%====================================================================
function avec=bpweight(up,dn,K);
%  THIS IS A MODIFIED VERSION OF BAXTER AND KING'S BPF.M FILE
% 
% Program to compute band-pass filtered series
% Inputs are
%  up:  period corresponding to highest frequency (e.g., 6)
%  dn:  period corresponding to lowest frequency (e.g., 32)
%  K:   number of terms in approximating moving average

% Implied Frequencies
omubar=2*pi/up; omlbar=2*pi/dn;

% An approximate low pass filter, with a cutoff frequency of "ombar",
% has a frequency response function 
%
%  alpha(om) = a0 + 2*a1 cos(om) + ... 2*aK cos(K om)
%
% and the ak's are given by:
%  
% a0 = ombar/(pi)             ak = sin(k ombar)/(k pi)
%
% where ombar is the cutoff frequency.

% A band-pass filter is the difference between two 
% low-pass filters,
%   bp(L) = bu(L) - bl(L)
% with bu(L) being the filter with the high cutoff point and bl(L) being 
% that with the low cutoff point.  Thus, the weights are differences
% of weights for two low-pass filters.
% Construct filter weights for bandpass filter (a(0)....a(K)). 

akvec=zeros(1,K+1);
akvec(1)=(omubar-omlbar)/(pi);  % weight at k=0
for k=1:K;
  akvec(k+1)=(sin(k*omubar)-sin(k*omlbar))/(k*pi); % weights at k=1,2,...K 
end

% Impose constraint on frequency response at om = 0 
% (If high pass filter, this amounts to requiring that weights sum to zero).
% (If low pass filter, this amounts to requiring that weights sum to one).

% sum of weights without constraint
theta=akvec(1)+2*sum(akvec(2:K+1));

%/* amount to add to each weight to get sum to add to zero */
if (omlbar > 1e-8); theta = -theta/(2*(K+1)); else; theta = (1-theta)/(2*(K+1)); end;
akvec=akvec+theta;
akvec(1)=akvec(1)+theta;

%@ Set vector of weights @
avec=zeros(2*K+1,1);
avec(K+1)=akvec(1);
for i=1:K;
    avec(K+1-i)=akvec(i+1);
    avec(K+1+i)=akvec(i+1);
end;

