function [AF,PF,AS,PS,mu,F]=getKlmNruProd(theta,y,X,Z,unLags);
%
% filtered and smoothed estimates of the state variable and its variance 
%
% measurement: y_t = z_t'a_t + d_t + e_t;   d_t=X_t*gamma; var(e) = H
% transition:  a_t = T*a_t-1 + c_t + u_t;   c_t=Z_t'b;     var(u) = lambda*H = Q
%
% gamma: theta(1:cols(X)),   H theta(cols(X)+1),  b theta(cols(X)+2:end-1); Q(1) theta(end) 

theta=makeCol(theta); nobs=length(y);
gmma=theta(1:size(X,2)); H=theta(size(X,2)+1); b=theta(size(X,2)+2:end-1);
u=y-X*gmma; % d_t=X*gmma

z=-theta(2:unLags+1); 
T=zeros(unLags); T(1)=1; c=Z*b;
Q=zeros(unLags); Q(1)=theta(end);
if unLags>1; T(2:end,1:end-1)=eye(unLags-1); end;

mu=repmat(nan,nobs,1); F=mu; atgtm1=repmat(nan,unLags,nobs); atgt=atgtm1; 
ptgtm1=repmat(nan,[unLags,unLags,nobs]); ptgt=ptgtm1;  
for t = 1:nobs;
    % Prediction
    if t==1; atgtm1(:,t)=repmat(0,unLags,1); ptgtm1(:,:,t)=zeros(unLags); ptgtm1(1,1,1)=1; 
    else; 
        atgtm1(:,t)=T*atgt(:,t-1) + [c(t);zeros(unLags-1,1)]; ptgtm1(:,:,t)=T*ptgt(:,:,t-1)*T'+Q;
    end;
    % Updating
    mu(t) = u(t) - z'*atgtm1(:,t);
    F(t) = z'*ptgtm1(:,:,t)*z+H;
    atgt(:,t) = atgtm1(:,t)+ptgtm1(:,:,t)*z*pinv(F(t))*mu(t);
    ptgt(:,:,t) = ptgtm1(:,:,t)*( eye(unLags)-z*pinv(F(t))*z'*ptgtm1(:,:,t) );
end;

% Smoother
atgT(:,nobs)=atgt(:,nobs); ptgT(:,:,nobs)=ptgt(:,:,nobs);
for t= 1:nobs-1;
    J(:,:,nobs-t)=ptgt(:,:,nobs-t)*T'*pinv(ptgtm1(:,:,nobs-t+1));
    atgT(:,nobs-t) = atgt(:,nobs-t)+J(:,:,nobs-t)*(atgT(:,nobs-t+1)-atgtm1(:,nobs-t+1)-[c(nobs-t);zeros(unLags-1,1)]);
    ptgT(:,:,nobs-t) = ptgt(:,:,nobs-t)+J(:,:,nobs-t)*( ptgT(:,:,nobs-t+1)-ptgtm1(:,:,nobs-t+1) )*J(:,:,nobs-t)';
end;

AF=atgt(1,:)'+c(end-length(X)+1:end)+theta(1)/sum(z);
PF=squeeze(ptgt(1,1,:));
AS=atgT(1,:)'+c(end-length(X)+1:end)+theta(1)/sum(z);
PS=squeeze(ptgT(1,1,:));
%muT=u-atgT*z;  