function L=klmNru(theta,y,X,unLags,lambda);
%
% log-likelihood function for the NAIRU, Harvey's notation
%
% measurement: y_t = z'a_t + d_t + e_t;   d_t=X_t*gamma; var(e) = H
% transition:  a_t = T*a_t-1 + u_t;                      var(u) = lambda*H = Q
%
% theta contains the regressions coeffs and a variance at the end
% 
theta=makeCol(theta); nobs=length(y);
u=y-X*theta(1:end-1);

Z=-theta(2:unLags+1);
T=zeros(unLags); T(1)=1;
H=theta(end); Q=zeros(unLags); Q(1)=lambda*theta(end);
if unLags>1; T(2:end,1:end-1)=eye(unLags-1); end;

mu=repmat(nan,nobs,1); F=mu; atgtm1=repmat(nan,unLags,nobs); atgt=atgtm1; 
ptgtm1=repmat(nan,[unLags,unLags,nobs]); ptgt=ptgtm1;  
for t = 1:nobs;
    % Prediction
    if t==1; atgtm1(:,t)=repmat(0,unLags,1); ptgtm1(:,:,t)=zeros(unLags); ptgtm1(1,1,1)=1; 
    else; atgtm1(:,t)=T*atgt(:,t-1); ptgtm1(:,:,t)=T*ptgt(:,:,t-1)*T'+Q; %...+Q or +lambda*H
    end;
    % Updating
    mu(t) = u(t) - Z'*atgtm1(:,t);
    F(t) = Z'*ptgtm1(:,:,t)*Z+H;
    atgt(:,t) = atgtm1(:,t)+ptgtm1(:,:,t)*Z*pinv(F(t))*mu(t);
    ptgt(:,:,t) = ptgtm1(:,:,t)*( eye(unLags)-Z*pinv(F(t))*Z'*ptgtm1(:,:,t) );
end;

L = .5*sum( log(F) + mu.^2./F );
