function L=klmNru(theta,y,X,lambda);
%
% assumption: just 1 lag of u !!
%
% log-likelihood function for the NAIRU, Harvey's notation
%
% measurement: y_t = z'a_t + d_t + e_t;   d_t=X_t*gamma; var(e) = H
% transition:  a_t = T*a_t-1 + u_t;                      var(u) = lambda*H = Q
%
% theta contains the regressions coeffs and a variance at the end
% 
theta=makeCol(theta); nobs=length(y);
u=y-X*theta(1:end-1);

Z=zeros(2); Z(1)=-theta(2);
T=eye(2); T(3)=1;
H=theta(end); Q=zeros(unLags); Q=zeros(2); Q(1)=lambda(1)*theta(end); Q(4)=lambda(2)*theta(end);

mu=repmat(nan,nobs,1); F=mu; atgtm1=repmat(nan,2,nobs); atgt=atgtm1; 
ptgtm1=repmat(nan,[2,2,nobs]); ptgt=ptgtm1;  
for t = 1:nobs;
    % Prediction
    if t==1; atgtm1(:,t)=repmat(0,2,1); ptgtm1(:,:,t)=zeros(2); ptgtm1(:,:,1)=eye(2); 
    else; atgtm1(:,t)=T*atgt(:,t-1); ptgtm1(:,:,t)=T*ptgt(:,:,t-1)*T'+Q; %...+Q or +lambda*H
    end;
    % Updating
    mu(t) = u(t) - Z'*atgtm1(:,t);
    F(t) = Z'*ptgtm1(:,:,t)*Z+H;
    atgt(:,t) = atgtm1(:,t)+ptgtm1(:,:,t)*Z*pinv(F(t))*mu(t);
    ptgt(:,:,t) = ptgtm1(:,:,t)*( eye(2)-Z*pinv(F(t))*Z'*ptgtm1(:,:,t) );
end;

L = .5*sum( log(F) + mu.^2./F );
