function [QLR,QLRR]=myQLR(y,x1,x2,trim,nlags,kernel,level,plotIndex,time);
% Note: need to finish p values for mean and exp tests
% [QLR,QLRR]=myQLR(y,x1,x2,trim,nlags,kernel,level);
% AFTER SERENA NG'S M FILE
% program performs the QLR (sup Wald) test for parameter stability
% y is the left hand side variable
% x1 is the matrix of regressors whose coefficients do not change over time
%      if all coefficients change, enter an empty matrix []
% x2 is the matrix of regressors whose coefficients are constant under
%      under the null but is time varying under the alternative
% trim is percent of the sample in the beginning and the end to exclude
%      when doing the sequential tests
% nlags =0 uses White's standard errors (no autocorrelation)
% nlags between 1 and 99 fixes the lag length for correcting serial correlation
% nlags =-1 does automatic bandwidth selection
% kernel == 1 is parzen
%        == 2 is quadratic spectral
%        == 3 is Newey and west;
% level is the significance level for constructing confidence intervals
% QLRR is the robust test
% QLR is the test under homoskedasticity and no serial correlation
% QLR(1): test statistic
% QLR(2): break date
% QLR(3): lower 1-level CI
% QLR(4): upper 1-level CI
% QLR(5): pvalue (for sup F)
%
y=makeCol(y); T=length(y); tt=(1:T)';
T1=floor(trim*T); T2=T-T1;
if nargin == 7; plotIndex=0; time=tt; end; if nargin==8; time=tt; end;
Wald=zeros(T,1); WaldR=zeros(T,1);
x=x2;

if ~isempty(x1) ; x=[x1 x2]; end;
K1=size(x1,2); K2=size(x2,2); K=K1+K2;

for t=T1:T2;
	DU=ones(T,1).*(tt> t);
	z=x2.*repmat(DU,1,K2);
	reg=[x z];
	beta=pinv(reg'*reg)*reg'*y;               % the unrestricted estimates
	e1=y-reg*beta; s2=e1'*e1/(T-(K+K2));
	mxx_inv=inv((reg'*reg)/T);
	beta22=beta(K+1:K+K2);
	vbeta=s2*mxx_inv/T;              % standard OLS variance
	vbeta22=vbeta(K+1:K+K2,K+1:K+K2);
	
    % do the HAC correction
	we=reg.*repmat(e1,1,size(reg,2));
	[vcv,k]=hac(we,nlags,0,kernel);
	r_vbeta=mxx_inv*(vcv/T)*mxx_inv;  % robust variance
	r_vbeta22=r_vbeta(K+1:K+K2,K+1:K+K2);
	Wald(t)=beta22'*(pinv(vbeta22))*beta22;
	WaldR(t)=beta22'*(pinv(r_vbeta22))*beta22;
end;

% sup F (Andrews 92), exp F and ave F tests (Andrews and Ploberger 93) 
[QLR QLRT] =max(Wald); [QLRR QLRRT]=max(WaldR);
expF=log(mean(exp(.5*Wald))); expFR=log(mean(exp(.5*WaldR))); 
aveF=mean(Wald); aveFR=mean(WaldR);

[TB1,TB2]=qlrCI(y,[x1 x2],QLRT,nlags,0,kernel,level);
[TB1_r,TB2_r]=qlrCI(y,[x1 x2],QLRRT,nlags,0,kernel,level);

QLR=[QLR QLRT TB1 TB2 pValQLR(QLR,K2)];
QLRR=[QLRR QLRRT TB1_r TB2_r pValQLR(QLRR,K2)];
if plotIndex; 
    % 95% cvs from Andrews(1993), tab 1
    q95=[8.85 11.79 14.15 16.45 18.35 20.26 21.84 22.13 25.47 27.03 28.55 30.16 31.80 33.45 35.06 36.66 38.12 39.55 41.25 43.00];
    figure; h=myPlot(time,[Wald WaldR ones(size(Wald))*q95(K2)]); set(h,'linewidth',2); 
    legend('Wald Statistic','Robust Wald Statistic','95% critical value'); title(['sup F tests']); 
end;

%=========================================================================
function pVal=pValQLR(x,n);
% AFTER STOCK AND WATSON'S GAUSS PROCEDURE 2/13/03
% @ -------------------------------------------------------------- @
% proc(1) = pval_qlr(x,n,ccut);
% /* pval_qlr.prc, mww, 1/27/97
%    Calculate pvalue for QLR tests.
%    x = value of test statistic
%    n = number of degrees of freedon
%    ccut = trimming constant
% 
%    Pvalue is computed using the approximation in
%    Hansen, 1995, JBES, "Approximate Asymptotic P Values ... "
%    pages 60-67.
% 
% */

parmvec =[...
	1    -0.99 1.02  3.0;...
	2    -1.65 1.06  4.7;...
	3    -2.05 1.13  6.8;...
	4    -2.52 1.11  8.0;...
	5    -3.46 1.07  8.3;...
	6    -4.05 1.08  9.5;...
	7    -4.42 1.10 11.0;...
	8    -5.36 1.08 11.3;...
	9    -5.43 1.10 13.1;...
	10   -6.47 1.06 12.8;...
	11   -6.79 1.04 13.5;...
	12   -7.80 1.02 13.6;...
	13   -7.93 1.07 15.9;...
	14   -8.54 1.05 16.1;...
	15   -9.05 1.05 17.2;...
	16   -9.13 1.09 19.3;...
	17  -10.45 1.05 18.3;...
	18  -10.63 1.05 19.5;...
	19  -12.14 0.90 14.9;...
	20  -12.14 0.97 18.3;...
	25  -14.16 1.05 25.0;...
	30  -17.06 1.03 27.8;...
	35  -20.09 0.98 28.4;...
	40  -21.65 1.05 36.7];

%@ -- Find Appropriate Row of Parmvec -- @
d=abs(parmvec(:,1)-n);
[di,diInd]=min(d);
if di > .0001; pVal=nan; return; 
else;
	p=parmvec(diInd,:);
	x1=p(1,2)+p(1,3)*x;
	if x1 >= 0; pVal=chi2cdf(x1,p(1,4));
	else; pVal=1;
	end;
end;
pVal=1-pVal;
