function Vmat=nwest_mine(x,e,nlag)
% PURPOSE: computes Newey-West adjusted heteroscedastic-serial
%          consistent Least-squares Regression
%---------------------------------------------------
% USAGE: results = nwest(y,x,nlag)
% where: y = dependent variable vector (nobs x 1)
%        x = independent variables matrix (nobs x nvar)
%     nlag = lag length to use
%---------------------------------------------------
% RETURNS: a structure
%        results.meth  = 'newlyw'
%        results.beta  = bhat
%        results.tstat = t-stats
%        results.yhat  = yhat
%        results.resid = residuals
%        results.sige  = e'*e/(n-k)
%        results.rsqr  = rsquared
%        results.rbar  = rbar-squared
%        results.dw    = Durbin-Watson Statistic
%        results.nobs  = nobs
%        results.nvar  = nvars
%        results.y     = y data vector
%        results.nwerr = Newey-West standard errors   
%        results.V     = NW who;e var matrix
% --------------------------------------------------
% SEE ALSO: nwest_d, prt(results), plt(results)
%---------------------------------------------------
% References:  Gallant, R. (1987),
%  "Nonlinear Statistical Models," pp.137-139.
%---------------------------------------------------

% written by:
% James P. LeSage, Dept of Economics
% University of Toledo
% 2801 W. Bancroft St,
% Toledo, OH 43606
% % jlesage@spatial-econometrics.com
% 

% perform Newey-West correction
% e T x m matrix of residuals
% x T x m matrix of regressors, no constant 

[nobs,m]=size(e); Vmat=repmat(nan,2*m,2*m); ii=1;

for i=1:m;
    jj=1;
    for j=1:m; 
        xO=[ones(size(x(:,i))) x(:,i)]; xT=[ones(size(x(:,j))) x(:,j)];
        xIpxJi = inv(xO'*xT);
        ematO = repmat(e(:,i)',2,1); ematT = repmat(e(:,j)',2,1);   
        hhatO=ematO.*xO'; hhatT=ematT.*xT';
        G=zeros(2); w=zeros(2*nlag+1,1);
        a=0;

        while a~=nlag+1;
            ga=zeros(2);
            w(nlag+1+a,1)=(nlag+1-a)/(nlag+1);
            za=hhatO(:,(a+1):nobs)*hhatT(:,1:nobs-a)';
              if a==0;
               ga=ga+za;
              else
               ga=ga+za+za';
              end;
            G=G+w(nlag+1+a,1)*ga;
            a=a+1;
        end; % end of while
    
        V=xIpxJi*G*xIpxJi;
        Vmat(ii:ii+1,jj:jj+1)=V;
        jj=jj+2;
    end;
    ii=ii+2;
end;
